/*
 * Simulator of microcontrollers (uc.cc)
 *
 * Copyright (C) 1999,99 Drotos Daniel, Talker Bt.
 * 
 * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
 *
 */

/* This file is part of microcontroller simulator: ucsim.

UCSIM is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UCSIM is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UCSIM; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */
/*@1@*/

#include "ddconfig.h"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include "i_string.h"

#include "uccl.h"
#include "hwcl.h"
#include "memcl.h"
#include "simcl.h"


/*
 * Abstract ucrocontroller
 */

cl_uc::cl_uc(class cl_sim *asim):
  cl_base()
{
  sim = asim;
  mems= new cl_list(MEM_TYPES, 1);
  hws = new cl_list(1, 1);
  options= new cl_list(2, 2);
}


cl_uc::~cl_uc(void)
{
  delete mems;
  delete hws;
  delete options;
}


int
cl_uc::init(void)
{
  int mc;
  cl_base::init();
  for (mc= MEM_ROM; mc < MEM_TYPES; mc++)
    mk_mem((enum mem_class)mc);
  /*mk_rom();
  mk_xram();
  mk_iram();
  mk_sfr();
  mk_hw();*/
  return(0);
}


/* Making variables */

int
cl_uc::mk_mem(enum mem_class type)
{
  class cl_mem *m= new cl_mem(type, get_mem_size(type));
  mems->add_at(type, m);
  return(m != 0);
}

long
cl_uc::get_mem_size(enum mem_class type)
{
  switch (type)
    {
    case MEM_ROM: return(0x10000);
    case MEM_XRAM: return(0x10000);
    case MEM_IRAM: return(0x100);
    case MEM_SFR: return(0x100);
    }
  return(0);
}


/* Read/write simulated memory */

ulong
cl_uc::read_mem(enum mem_class type, long addr)
{
  class cl_mem *m;

  if (m= (class cl_mem*)mems->at(type))
    return(m->read(addr));
printf("TROUBLE\n");
  return(0);
}

ulong
cl_uc::get_mem(enum mem_class type, long addr)
{
  class cl_mem *m;

  if (m= (class cl_mem*)mems->at(type))
    return(m->get(addr));
printf("TROUBLE\n");
  return(0);
}

void
cl_uc::write_mem(enum mem_class type, long addr, ulong val)
{
  class cl_mem *m;

  if (m= (class cl_mem*)mems->at(type))
    {
      m->write(addr, &val);
      //m->mem[addr]= val;
    }
else printf("TROUBLE\n");
}

void
cl_uc::set_mem(enum mem_class type, long addr, ulong val)
{
  class cl_mem *m;

  if (m= (class cl_mem*)mems->at(type))
    m->set(addr, val);
else printf("TROUBLE\n");
}

class cl_mem *
cl_uc::mem(enum mem_class type)
{
  if (mems->count < type)
{printf("TROUBLE\n");    return(0);
}
  return((class cl_mem *)(mems->at(type)));
}

uchar *
cl_uc::MEM(enum mem_class type)
{
  class cl_mem *m;

  if ((m= mem(type)) == 0)
{printf("TROUBLE\n");    return(0);
}
  return(m->mem);
}


/* Local function for `read_hex_file' method to read some bytes */

static long
ReadInt(FILE *f, bool *ok, int bytes)
{
  char s2[3];
  long l= 0;

  *ok= FALSE;
  while (bytes)
    {
      if (fscanf(f, "%2c", &s2[0]) == EOF)
	return(0);
      s2[2]= '\0';
      l= l*256 + strtol(s2, NULL, 16);
      bytes--;
    }
  *ok= TRUE;
  return(l);
}


/* 
 * Reading intel hexa file into EROM
 *____________________________________________________________________________
 *
 * If parameter is a NULL pointer, this function reads data from `cmd_in'
 *
 */

long
cl_uc::read_hex_file(const char *name)
{
  FILE *f;
  char c;
  long written= 0, recnum= 0;

  uchar dnum;     // data number
  uchar rtyp=0;   // record type
  uint  addr;     // address
  uchar rec[300]; // data record
  uchar sum ;     // checksum
  uchar chk ;     // check
  int  i;
  bool ok;

  if (!name)
    f= sim->/*FIXME*/cmd_in();
  else
    if ((f= fopen(name, "r")) == NULL)
      {
	fprintf(stderr, "Can't open `%s': %s\n", name, strerror(errno));
	return(-1);
      }

  //memset(inst_map, '\0', sizeof(inst_map));
  ok= TRUE;
  while (ok &&
	 rtyp != 1)
    {
      while (((c= getc(f)) != ':') &&
	     (c != EOF)) ;
      if (c != ':')
	{printf(": not found\n");break;}
      recnum++;
      dnum= ReadInt(f, &ok, 1);//printf("dnum=%02x",dnum);
      chk = dnum;
      addr= ReadInt(f, &ok, 2);//printf("addr=%04x",addr);
      chk+= (addr & 0xff);
      chk+= ((addr >> 8) & 0xff);
      rtyp= ReadInt(f, &ok, 1);//printf("rtyp=%02x ",rtyp);
      chk+= rtyp;
      for (i= 0; ok && (i < dnum); i++)
	{
	  rec[i]= ReadInt(f, &ok, 1);//printf("%02x",rec[i]);
	  chk+= rec[i];
	}
      if (ok)
	{
	  sum= ReadInt(f, &ok, 1);//printf(" sum=%02x\n",sum);
	  if (ok)
	    {
	      if (((sum + chk) & 0xff) == 0)
		{
		  if (rtyp == 0)
		    for (i= 0; i < dnum; i++)
		      {
			set_mem(MEM_ROM, addr, rec[i]);
			addr++;
			written++;
		      }
		  else
		    if (sim->get_iarg('V', 0) &&
			rtyp != 1)
		      fprintf(sim->cmd_out(),
			      "Unknown record type %d(0x%x)\n", rtyp);
		}
	      else
		if (sim->get_iarg('V', 0))
		  fprintf(sim->cmd_out(),
			  "Checksum error (%x instead of %x) in record %ld.\n",
			  chk, sum, recnum);
	    }
	  else
	    if (sim->get_iarg('V', 0))
	      fprintf(sim->cmd_out(), "Read error in record %ld.\n", recnum);
	}
    }
  if (name)
    fclose(f);
  if (sim->get_iarg('V', 0))
    fprintf(sim->cmd_out(), "%ld records have been read\n", recnum);
  analyze(0);
  return(written);
}


/* Register callback hw objects for mem read/write */

void
cl_uc::register_hw_read(enum mem_class type, long addr, class cl_hw *hw)
{
  class cl_mem *m;
  class cl_memloc *l;

  if (m= (class cl_mem*)mems->at(type))
    {
      if ((l= m->read_locs->get_loc(addr)) == 0)
	{
	  l= new cl_memloc(addr);
	  l->init();
	  m->read_locs->add(l);
	}
      l->hws->add(hw);
    }
}

void
cl_uc::register_hw_write(enum mem_class type, long addr, class cl_hw *hw)
{
}

struct dis_entry *
cl_uc::dis_tbl(void)
{
  return(0);
}

struct name_entry *
cl_uc::sfr_tbl(void)
{
  return(0);
}

struct name_entry *
cl_uc::bit_tbl(void)
{
  return(0);
}


/* End of uc.cc */
